/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gesture for Mozilla.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *  Mike Kozlowski <mozdev@klio.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var trailDot;
var trailArea;
var trailActive = false;
var trailLength;
var trailInt;
var trailX, trailY;
var trailXBoundary, trailYBoundary;
var trailCache = new Array();

function shouldDrawTrail(startElement) {
  if (!drawTrail) return false;
  var cur = startElement;
  while (cur != null && cur != mgGetContentArea()) {
    if (cur.nodeName.toLowerCase() == "html") return true;
    cur = cur.parentNode;
  }
  return false;
}

function mozgestBeginTrail(e){
  if (shouldDrawTrail(e.target)) {
    var doc = e.target.ownerDocument;
    trailArea = doc.createElementNS("http://www.w3.org/1999/xhtml", "div");
    var body = doc.getElementsByTagName("body")[0];
    body.appendChild(trailArea);

    trailDot = doc.createElementNS("http://www.w3.org/1999/xhtml", "div");
    trailDot.style.width = trailWidth+"px";
    trailDot.style.height = trailWidth+"px";
    trailDot.style.backgroundColor = trailColor;
    if (trailOpacity != 100) {
      trailDot.style.MozOpacity = trailOpacity/100;
    }
    trailDot.style.position = "absolute";
    trailX = e.pageX;
    trailY = e.pageY;

    if (e.target.ownerDocument.body) {
      trailXBoundary = e.view.pageXOffset + e.target.ownerDocument.body.clientWidth;
      trailYBoundary = e.view.pageYOffset + e.target.ownerDocument.body.clientHeight;
    } else {
      trailXBoundary = null;
      trailYBoundary = null;
    }
    trailDot.style.zIndex = "100";
    trailLength = trailInt = 0;
    trailActive = true;
  }
}

function mozgestDrawTrail(e){
  if (!trailActive || trailLength > trailLimit) return;

  var dot;
  if (gesture.length == 1 && trailCache.length != 0) {
    for (var i = 0; i < trailCache.length; i++) {
      trailArea.appendChild(trailCache[i]);
    }
  }
  xMove = e.pageX-trailX;
  yMove = e.pageY-trailY;
  var xDecrement = (xMove < 0) ? 1 : (-1);
  var yDecrement = (yMove < 0) ? 1 : (-1);
  if (Math.abs(xMove) >= Math.abs(yMove) && xMove != 0) {
    for (x = xMove; x != 0; x = x + xDecrement) {
      mozgestSetDot(e.pageX - x, e.pageY - Math.round(yMove * x / xMove), e);
    }
  } else {
    for (y = yMove; y != 0; y = y + yDecrement) {
      mozgestSetDot(e.pageX - Math.round(xMove * y / yMove), e.pageY - y, e);
    }
  }
  trailX = e.pageX;
  trailY = e.pageY;
}

function mozgestSetDot(x, y, e) {
  if (trailInt % trailInterval == 0) {
    var h = Math.round(trailWidth/2);
    
    dot = trailDot.cloneNode(true);

    // if a dot would extend into scrollbar or window border, make it smaller
    // to avoid appearing scrollbars. do not draw dots outside of client area.
    if (trailXBoundary && trailYBoundary) {
      var rSpace = trailXBoundary - (x-h+trailWidth);
      if (rSpace < 0) {
        if (trailWidth + rSpace < 0) return;
        dot.style.width = (trailWidth + rSpace) + "px";
      }
      var dSpace = trailYBoundary - (y-h+trailWidth);
      if (dSpace < 0) {
        if (trailWidth + dSpace < 0) return;
        dot.style.height = (trailWidth + dSpace) + "px";
      }
    }

    dot.style.top = (y-h)+"px";
    dot.style.left = (x-h)+"px";
    if (gesture.length > 0) {
      trailArea.appendChild(dot);
    } else {
      trailCache.push(dot);
    }
    trailLength++;
  }
  trailInt++;
}

function mozgestEndTrail() {
  if (trailArea && trailArea.parentNode) {
    trailArea.parentNode.removeChild(trailArea);
  }
  trailCache.length = 0;
  trailArea = false;
  trailActive = false;
}